from typing import List, Dict, Any, Tuple
import time
import numpy as np
import csle_collector.constants.constants as constants
from csle_collector.snort_ids_manager.dao.snort_ids_alert import SnortIdsFastLogAlert
import csle_collector.snort_ids_manager.snort_ids_manager_pb2


class SnortIdsAlertCounters:
    """
    DTO containing statistics from the Snort IDS log
    """

    def __init__(self):
        """
        Initializes the DTO
        """
        self.priority_alerts = [0] * 4
        self.class_alerts = []
        for i in range(len(set(constants.SNORT_IDS_ROUTER.SNORT_ALERT_IDS_ID.values()))):
            self.class_alerts.append(0)
        self.severe_alerts = 0
        self.warning_alerts = 0
        self.total_alerts = 0
        self.alerts_weighted_by_priority = 0
        self.ip = ""
        self.ts = 0.0

    def add(self, alert_counters: "SnortIdsAlertCounters") -> None:
        """
        Adds another alert counters object to this one

        :param alert_counters: the counters to add
        :return: None
        """
        self.severe_alerts = int(self.severe_alerts + alert_counters.severe_alerts)
        self.warning_alerts = int(self.warning_alerts + alert_counters.warning_alerts)
        self.total_alerts = int(self.total_alerts + alert_counters.total_alerts)
        self.alerts_weighted_by_priority = self.alerts_weighted_by_priority + alert_counters.alerts_weighted_by_priority
        for idx in range(len(self.priority_alerts)):
            self.priority_alerts[idx] = int(self.priority_alerts[idx] + alert_counters.priority_alerts[idx])
        for idx in range(len(self.class_alerts)):
            self.class_alerts[idx] = int(self.class_alerts[idx] + alert_counters.class_alerts[idx])

    def count(self, alerts: List[SnortIdsFastLogAlert]) -> None:
        """
        Counts the list of alerts

        :param alerts: list of alerts from the log
        :return: None
        """
        for a in alerts:
            if a.priority - 1 in range(0, len(self.priority_alerts)):
                self.priority_alerts[a.priority] += 1
            if a.class_id in range(0, len(self.class_alerts)):
                self.class_alerts[a.class_id] += 1

        self.total_alerts = len(alerts)
        self.severe_alerts = sum(
            self.priority_alerts[0:constants.SNORT_IDS_ROUTER.SNORT_SEVERE_ALERT_PRIORITY_THRESHOLD])
        self.warning_alerts = sum(
            self.priority_alerts[constants.SNORT_IDS_ROUTER.SNORT_SEVERE_ALERT_PRIORITY_THRESHOLD:])
        self.alerts_weighted_by_priority = 0
        for idx in range(len(self.priority_alerts)):
            priority = (len(self.priority_alerts) - idx + 1)
            self.alerts_weighted_by_priority += priority * self.priority_alerts[idx]

    @staticmethod
    def from_kafka_record(record: str) -> "SnortIdsAlertCounters":
        """
        Converts a kafka record to a DTO

        :param record: the kafka record to convert
        :return: the DTO
        """
        parts = record.split(",")
        obj = SnortIdsAlertCounters()
        obj.ts = float(parts[0])
        obj.ip = parts[1]
        obj.total_alerts = int(round(float(parts[2])))
        obj.warning_alerts = int(round(float(parts[3])))
        obj.severe_alerts = int(round(float(parts[4])))
        obj.alerts_weighted_by_priority = int(round(float(parts[5])))
        obj.class_alerts = []
        obj.priority_alerts = []
        for i in range(6, len(set(constants.SNORT_IDS_ROUTER.SNORT_ALERT_IDS_ID.values())) + 6):
            obj.class_alerts.append(int(round(float(parts[i]))))
        for i in range(len(set(constants.SNORT_IDS_ROUTER.SNORT_ALERT_IDS_ID.values())) + 6,
                       len(set(constants.SNORT_IDS_ROUTER.SNORT_ALERT_IDS_ID.values())) + 10):
            obj.priority_alerts.append(int(round(float(parts[i]))))
        return obj

    def update_with_kafka_record(self, record: str) -> None:
        """
        Updates the DTO with a kafka record

        :param record: the kafka record to use for updating
        :return: None
        """
        parts = record.split(",")
        self.ts = float(parts[0])
        self.ip = parts[1]
        self.total_alerts = int(round(float(parts[2])))
        self.warning_alerts = int(round(float(parts[3])))
        self.severe_alerts = int(round(float(parts[4])))
        self.alerts_weighted_by_priority = int(round(float(parts[5])))

        self.class_alerts = []
        self.priority_alerts = []
        for i in range(6, len(set(constants.SNORT_IDS_ROUTER.SNORT_ALERT_IDS_ID.values())) + 6):
            self.class_alerts.append(int(round(float(parts[i]))))
        for i in range(len(set(constants.SNORT_IDS_ROUTER.SNORT_ALERT_IDS_ID.values())) + 6,
                       len(set(constants.SNORT_IDS_ROUTER.SNORT_ALERT_IDS_ID.values())) + 10):
            self.priority_alerts.append(int(round(float(parts[i]))))

    def to_kafka_record(self, ip: str) -> str:
        """
        Converts the DTO into a kafka record

        :param ip: the ip to add to the record in addition to the IDS statistics
        :return: a comma-separated string representing the kafka record
        """
        ts = time.time()
        total_counters = [ts, ip, self.total_alerts, self.warning_alerts, self.severe_alerts,
                          self.alerts_weighted_by_priority] + self.class_alerts + self.priority_alerts
        record_str = ",".join(list(map(lambda x: str(x), total_counters)))
        return record_str

    def to_dto(self, ip: str) -> csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsLogDTO:
        """
        Converts the object into a gRPC DTO for serialization

        :param ip: the ip to add to the DTO in addition to the statistics
        :return: A csle_collector.snort_ids_manager.snort_ids_manager_pb2.IdsLogDTOb
        """
        ts = time.time()
        return csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsLogDTO(
            timestamp=ts,
            ip=ip,
            attempted_admin_alerts=int(self.class_alerts[33]),
            attempted_user_alerts=int(self.class_alerts[32]),
            inappropriate_content_alerts=int(self.class_alerts[31]),
            policy_violation_alerts=int(self.class_alerts[30]),
            shellcode_detect_alerts=int(self.class_alerts[29]),
            successful_admin_alerts=int(self.class_alerts[28]),
            successful_user_alerts=int(self.class_alerts[27]),
            trojan_activity_alerts=int(self.class_alerts[26]),
            unsuccessful_user_alerts=int(self.class_alerts[25]),
            web_application_attack_alerts=int(self.class_alerts[24]),
            attempted_dos_alerts=int(self.class_alerts[23]),
            attempted_recon_alerts=int(self.class_alerts[22]),
            bad_unknown_alerts=int(self.class_alerts[21]),
            default_login_attempt_alerts=int(self.class_alerts[20]),
            denial_of_service_alerts=int(self.class_alerts[19]),
            misc_attack_alerts=int(self.class_alerts[18]),
            non_standard_protocol_alerts=int(self.class_alerts[17]),
            rpc_portman_decode_alerts=int(self.class_alerts[16]),
            successful_dos_alerts=int(self.class_alerts[15]),
            successful_recon_largescale_alerts=int(self.class_alerts[14]),
            successful_recon_limited_alerts=int(self.class_alerts[13]),
            suspicious_filename_detect_alerts=int(self.class_alerts[12]),
            suspicious_login_alerts=int(self.class_alerts[11]),
            system_call_detect_alerts=int(self.class_alerts[10]),
            unusual_client_port_connection_alerts=int(self.class_alerts[9]),
            web_application_activity_alerts=int(self.class_alerts[8]),
            icmp_event_alerts=int(self.class_alerts[7]),
            misc_activity_alerts=int(self.class_alerts[6]),
            network_scan_alerts=int(self.class_alerts[5]),
            not_suspicious_alerts=int(self.class_alerts[4]),
            protocol_command_decode_alerts=int(self.class_alerts[3]),
            string_detect_alerts=int(self.class_alerts[2]),
            unknown_alerts=int(self.class_alerts[1]),
            tcp_connection_alerts=int(self.class_alerts[0]),
            priority_1_alerts=int(self.priority_alerts[0]),
            priority_2_alerts=int(self.priority_alerts[1]),
            priority_3_alerts=int(self.priority_alerts[2]),
            priority_4_alerts=int(self.priority_alerts[3]),
            total_alerts=self.total_alerts,
            warning_alerts=self.warning_alerts,
            severe_alerts=self.severe_alerts,
            alerts_weighted_by_priority=self.alerts_weighted_by_priority
        )

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return f"total_alerts: {self.total_alerts}, warning_alerts: {self.warning_alerts}, " \
               f"severe_alerts: {self.severe_alerts}, " \
               f"priority_alerts: {self.priority_alerts}, class_alerts: {self.class_alerts}," \
               f"alerts_weighted_by_priority: {self.alerts_weighted_by_priority}"

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "SnortIdsAlertCounters":
        """
        Converts a dict representaion of the object into an instance

        :param d: the dict to convert
        :return: the DTO
        """
        obj = SnortIdsAlertCounters()
        obj.ip = d["ip"]
        obj.ts = d["ts"]
        obj.total_alerts = d["total_alerts"]
        obj.warning_alerts = d["warning_alerts"]
        obj.severe_alerts = d["severe_alerts"]
        obj.priority_alerts = d["priority_alerts"]
        obj.class_alerts = d["class_alerts"]
        obj.alerts_weighted_by_priority = d["alerts_weighted_by_priority"]
        return obj

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation

        :return: a dict representation of the object
        """
        d: Dict[str, Any] = {}
        d["ip"] = self.ip
        d["ts"] = self.ts
        d["class_alerts"] = list(self.class_alerts)
        d["priority_alerts"] = list(self.priority_alerts)
        d["total_alerts"] = self.total_alerts
        d["warning_alerts"] = self.warning_alerts
        d["severe_alerts"] = self.severe_alerts
        d["alerts_weighted_by_priority"] = self.alerts_weighted_by_priority
        return d

    def copy(self) -> "SnortIdsAlertCounters":
        """
        :return: a copy of the object
        """
        c = SnortIdsAlertCounters()
        c.class_alerts = self.class_alerts
        c.priority_alerts = self.priority_alerts
        c.ip = self.ip
        c.ts = self.ts
        c.total_alerts = self.total_alerts
        c.warning_alerts = self.warning_alerts
        c.severe_alerts = self.severe_alerts
        c.alerts_weighted_by_priority = self.alerts_weighted_by_priority
        return c

    def get_deltas(self, counters_prime: "SnortIdsAlertCounters") -> Tuple[List[int], List[str]]:
        """
        Get the deltas between two counters objects

        :param counters_prime: the counters object to compare with
        :return: the deltas and the labels
        """
        deltas_priority = list(np.array(counters_prime.priority_alerts).astype(int).tolist())
        deltas_class = list(np.array(counters_prime.class_alerts).astype(int).tolist())
        deltas = ([int(counters_prime.total_alerts), int(counters_prime.warning_alerts),
                   int(counters_prime.severe_alerts), int(counters_prime.alerts_weighted_by_priority)]
                  + deltas_priority + deltas_class)
        labels = constants.KAFKA_CONFIG.SNORT_IDS_ALERTS_LABELS
        assert len(labels) == len(deltas)
        return list(deltas), labels

    def num_attributes(self) -> int:
        """
        :return: The number of attributes of the DTO
        """
        return 10 + len(set(constants.SNORT_IDS_ROUTER.SNORT_ALERT_IDS_ID.values()))

    @staticmethod
    def schema() -> "SnortIdsAlertCounters":
        """
        :return: get the schema of the DTO
        """
        return SnortIdsAlertCounters()



from typing import Any
import pytest
import pytest_mock
from subprocess import CompletedProcess
import csle_collector.ryu_manager.query_ryu_manager
from csle_collector.ryu_manager.ryu_manager_pb2 import RyuDTO
from csle_collector.ryu_manager.ryu_manager import RyuManagerServicer


class TestRyuManagerSuite:
    """
    Test suite for the Ryu manager
    """

    @pytest.fixture(scope='module')
    def grpc_add_to_server(self) -> Any:
        """
        Necessary fixture for pytest-grpc

        :return: the add_servicer_to_server function
        """
        from csle_collector.ryu_manager.ryu_manager_pb2_grpc import add_RyuManagerServicer_to_server
        return add_RyuManagerServicer_to_server

    @pytest.fixture(scope='module')
    def grpc_servicer(self) -> RyuManagerServicer:
        """
        Necessary fixture for pytest-grpc

        :return: the host manager servicer
        """
        return RyuManagerServicer()

    @pytest.fixture(scope='module')
    def grpc_stub_cls(self, grpc_channel):
        """
        Necessary fixture for pytest-grpc

        :param grpc_channel: the grpc channel for testing
        :return: the stub to the service
        """
        from csle_collector.ryu_manager.ryu_manager_pb2_grpc import RyuManagerStub
        return RyuManagerStub

    def test_getRyuStatus(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the getRyuStatus grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_ryu_status',
                     return_value=False)
        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_monitor_status',
                     return_value=False)
        response: RyuDTO = csle_collector.ryu_manager.query_ryu_manager.get_ryu_status(stub=grpc_stub)
        assert response is not None
        assert not response.monitor_running
        assert not response.ryu_running

        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_ryu_status',
                     return_value=True)
        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_monitor_status',
                     return_value=False)
        response = csle_collector.ryu_manager.query_ryu_manager.get_ryu_status(stub=grpc_stub)
        assert response is not None
        assert not response.monitor_running
        assert response.ryu_running

        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_ryu_status',
                     return_value=False)
        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_monitor_status',
                     return_value=True)
        response = csle_collector.ryu_manager.query_ryu_manager.get_ryu_status(stub=grpc_stub)
        assert response is not None
        assert not response.monitor_running
        assert not response.ryu_running

        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_ryu_status',
                     return_value=True)
        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_monitor_status',
                     return_value=True)
        response = csle_collector.ryu_manager.query_ryu_manager.get_ryu_status(stub=grpc_stub)
        assert response is not None
        assert response.monitor_running
        assert response.ryu_running

    def test_stopRyu(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopRyu grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_ryu_status',
                     return_value=True)
        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_monitor_status',
                     return_value=True)
        mocker.patch('subprocess.run', return_value=CompletedProcess(args="", returncode=0, stdout="testout",
                                                                     stderr="testerr"))
        response: RyuDTO = csle_collector.ryu_manager.query_ryu_manager.stop_ryu(stub=grpc_stub)
        assert response is not None
        assert not response.monitor_running
        assert not response.ryu_running

    def test_startRyu(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startRyu grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('time.sleep', return_value=None)
        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_ryu_status',
                     return_value=False)
        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_monitor_status',
                     return_value=False)
        mocker.patch('subprocess.run', return_value=CompletedProcess(args="", returncode=0, stdout="testout",
                                                                     stderr="testerr"))
        mocker.patch('subprocess.Popen', return_value=None)
        mocker.patch('csle_collector.ryu_manager.threads.failure_detector.FailureDetector.run',
                     return_value=True)
        mocker.patch('csle_collector.ryu_manager.threads.failure_detector.FailureDetector.__init__',
                     return_value=None)
        mocker.patch('csle_collector.ryu_manager.threads.failure_detector.FailureDetector.start',
                     return_value=True)
        port = 6060
        web_port = 8080
        controller = "controller"
        response: RyuDTO = csle_collector.ryu_manager.query_ryu_manager.start_ryu(stub=grpc_stub, port=port,
                                                                                  web_port=web_port,
                                                                                  controller=controller)
        assert response is not None
        assert not response.monitor_running
        assert response.ryu_running
        assert response.web_port == web_port
        assert response.port == port
        assert response.controller == controller

    def test_stopRyuMonitor(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopRyuMonitor grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_ryu_status',
                     return_value=True)
        mocker.patch('requests.post', return_value=True)
        response: RyuDTO = csle_collector.ryu_manager.query_ryu_manager.stop_ryu_monitor(stub=grpc_stub)
        assert response is not None
        assert not response.monitor_running
        assert response.ryu_running

    def test_startRyuMonitor(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startRyuMonitor grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        http_response_mock = mocker.MagicMock()
        http_response_mock.status_code = 200
        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_ryu_status',
                     return_value=False)
        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_monitor_status',
                     return_value=False)
        mocker.patch('requests.put', return_value=http_response_mock)
        kafka_ip = "192.168.5.5"
        kafka_port = 3333
        time_step_len = 30
        response: RyuDTO = csle_collector.ryu_manager.query_ryu_manager.start_ryu_monitor(
            stub=grpc_stub, kafka_ip=kafka_ip, kafka_port=kafka_port, time_step_len=time_step_len)
        assert response is not None
        assert not response.monitor_running
        assert not response.ryu_running
        assert response.kafka_ip == kafka_ip
        assert response.kafka_port == kafka_port
        assert response.time_step_len == time_step_len

        mocker.patch('csle_collector.ryu_manager.ryu_manager.RyuManagerServicer._get_ryu_status',
                     return_value=True)
        response = csle_collector.ryu_manager.query_ryu_manager.start_ryu_monitor(
            stub=grpc_stub, kafka_ip=kafka_ip, kafka_port=kafka_port, time_step_len=time_step_len)
        assert response is not None
        assert response.monitor_running
        assert response.ryu_running
        assert response.kafka_ip == kafka_ip
        assert response.kafka_port == kafka_port
        assert response.time_step_len == time_step_len

        http_response_mock = mocker.MagicMock()
        http_response_mock.status_code = 500
        mocker.patch('requests.put', return_value=http_response_mock)
        response = csle_collector.ryu_manager.query_ryu_manager.start_ryu_monitor(
            stub=grpc_stub, kafka_ip=kafka_ip, kafka_port=kafka_port, time_step_len=time_step_len)
        assert response is not None
        assert not response.monitor_running
        assert response.ryu_running
        assert response.kafka_ip == kafka_ip
        assert response.kafka_port == kafka_port
        assert response.time_step_len == time_step_len



from typing import List
import csle_collector.host_manager.host_manager_pb2_grpc
import csle_collector.host_manager.host_manager_pb2
import csle_collector.constants.constants as constants


def get_host_status(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Queries the Host manager for the status of the Host monitor thread

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    get_host_monitor_status_msg = \
        csle_collector.host_manager.host_manager_pb2.GetHostStatusMsg()
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO \
        = stub.getHostStatus(get_host_monitor_status_msg, timeout=timeout)
    return host_dto


def start_host_monitor(stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
                       kafka_ip: str, kafka_port: int, time_step_len_seconds: int,
                       timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to start the Host monitor thread

    :param kafka_ip: the ip of the Kafka server
    :param kafka_port: the port of the Kafka server
    :param time_step_len_seconds: the length of one time-step
    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    start_host_monitor_msg = csle_collector.host_manager.host_manager_pb2.StartHostMonitorMsg(
        kafka_ip=kafka_ip, kafka_port=kafka_port, time_step_len_seconds=time_step_len_seconds
    )
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.startHostMonitor(start_host_monitor_msg, timeout=timeout)
    return host_dto


def stop_host_monitor(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to stop the Host monitor thread

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    stop_host_monitor_msg = \
        csle_collector.host_manager.host_manager_pb2.StopHostMonitorMsg()
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.stopHostMonitor(stop_host_monitor_msg, timeout=timeout)
    return host_dto


def start_filebeat(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to start filebeat

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    start_filebeat_msg = \
        csle_collector.host_manager.host_manager_pb2.StartFilebeatMsg()
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO \
        = stub.startFilebeat(start_filebeat_msg, timeout=timeout)
    return host_dto


def stop_filebeat(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to stop filebeat

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    stop_filebeat_msg = \
        csle_collector.host_manager.host_manager_pb2.StopFilebeatMsg()
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.stopFilebeat(stop_filebeat_msg, timeout=timeout)
    return host_dto


def config_filebeat(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        log_files_paths: List[str], kibana_ip: str, kibana_port: int, elastic_ip: str,
        elastic_port: int, num_elastic_shards: int, kafka_topics: List[str], kafka_ip: str,
        kafka_port: int, filebeat_modules: List[str], reload_enabled: bool = False, kafka: bool = False,
        timeout=constants.GRPC.CONFIG_TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to update the filebeat configuration

    :param log_files_paths: the list of log files that filebeat should monitor
    :param kibana_ip: the IP of Kibana where the data should be visualized
    :param kibana_port: the port of Kibana where the data should be visualized
    :param elastic_ip: the IP of elastic where the data should be shipped
    :param elastic_port: the port of elastic where the data should be shipped
    :param num_elastic_shards: the number of elastic shards
    :param reload_enabled: whether automatic reload of modules should be enabled
    :param kafka: whether kafka should be added as input
    :param kafka_topics: list of kafka topics to ingest
    :param kafka_port: the kafka server port
    :param kafka_ip: the kafka server ip
    :param filebeat_modules: a list of filebeat modules to enable
    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    config_filebeat_msg = \
        csle_collector.host_manager.host_manager_pb2.ConfigFilebeatMsg(
            log_files_paths=log_files_paths, kibana_ip=kibana_ip, kibana_port=kibana_port, elastic_ip=elastic_ip,
            elastic_port=elastic_port, num_elastic_shards=num_elastic_shards, reload_enabled=reload_enabled,
            kafka=kafka, kafka_port=kafka_port, kafka_ip=kafka_ip, kafka_topics=kafka_topics,
            filebeat_modules=filebeat_modules)
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO \
        = stub.configFilebeat(config_filebeat_msg, timeout=timeout)
    return host_dto


def get_host_metrics(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub, failed_auth_last_ts: float,
        login_last_ts: float, timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.host_manager.host_manager_pb2.HostMetricsDTO:
    """
    Queries the Host manager for the data of the Host metrics from given timestamps

    :param stub: the stub to send the remote gRPC to the server
    :param failed_auth_last_ts: the timtestamp to parse failed login from
    :param login_last_ts: the timtestamp to parse the last login from
    :param timeout: the GRPC timeout (seconds)
    :return: an HostMetricsDTO with host metrics
    """
    get_host_metrics_msg = \
        csle_collector.host_manager.host_manager_pb2.GetHostMetricsMsg(
            failed_auth_last_ts=failed_auth_last_ts, login_last_ts=login_last_ts)
    host_metrics_dto: csle_collector.host_manager.host_manager_pb2.HostMetricsDTO = \
        stub.getHostMetrics(get_host_metrics_msg, timeout=timeout)
    return host_metrics_dto


def start_packetbeat(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to start packetbeat

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    start_packetbeat_msg = \
        csle_collector.host_manager.host_manager_pb2.StartPacketbeatMsg()
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.startPacketbeat(start_packetbeat_msg, timeout=timeout)
    return host_dto


def stop_packetbeat(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to stop packetbeat

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    stop_packetbeat_msg = \
        csle_collector.host_manager.host_manager_pb2.StopPacketbeatMsg()
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.stopPacketbeat(stop_packetbeat_msg, timeout=timeout)
    return host_dto


def config_packetbeat(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        kibana_ip: str, kibana_port: int, elastic_ip: str,
        elastic_port: int, num_elastic_shards: int,
        timeout=constants.GRPC.CONFIG_TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to update the packetbeat configuration

    :param kibana_ip: the IP of Kibana where the data should be visualized
    :param kibana_port: the port of Kibana where the data should be visualized
    :param elastic_ip: the IP of elastic where the data should be shipped
    :param elastic_port: the port of elastic where the data should be shipped
    :param num_elastic_shards: the number of elastic shards
    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    config_packetbeat_msg = \
        csle_collector.host_manager.host_manager_pb2.ConfigPacketbeatMsg(
            kibana_ip=kibana_ip, kibana_port=kibana_port, elastic_ip=elastic_ip,
            elastic_port=elastic_port, num_elastic_shards=num_elastic_shards)
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.configPacketbeat(config_packetbeat_msg, timeout=timeout)
    return host_dto


def start_metricbeat(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to start metricbeat

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    start_metricbeat_msg = \
        csle_collector.host_manager.host_manager_pb2.StartMetricbeatMsg()
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.startMetricbeat(start_metricbeat_msg, timeout=timeout)
    return host_dto


def stop_metricbeat(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to stop metricbeat

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    stop_metricbeat_msg = \
        csle_collector.host_manager.host_manager_pb2.StopMetricbeatMsg()
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.stopMetricbeat(stop_metricbeat_msg, timeout=timeout)
    return host_dto


def config_metricbeat(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        kibana_ip: str, kibana_port: int, elastic_ip: str,
        elastic_port: int, num_elastic_shards: int, kafka_ip: str,
        kafka_port: int, metricbeat_modules: List[str], reload_enabled: bool = False,
        timeout=constants.GRPC.CONFIG_TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to update the metricbeat configuration

    :param kibana_ip: the IP of Kibana where the data should be visualized
    :param kibana_port: the port of Kibana where the data should be visualized
    :param elastic_ip: the IP of elastic where the data should be shipped
    :param elastic_port: the port of elastic where the data should be shipped
    :param num_elastic_shards: the number of elastic shards
    :param reload_enabled: whether automatic reload of modules should be enabled
    :param kafka_port: the kafka server port
    :param kafka_ip: the kafka server ip
    :param metricbeat_modules: a list of metricbeat modules to enable
    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    config_metricbeat_msg = \
        csle_collector.host_manager.host_manager_pb2.ConfigMetricbeatMsg(
            kibana_ip=kibana_ip, kibana_port=kibana_port, elastic_ip=elastic_ip,
            elastic_port=elastic_port, num_elastic_shards=num_elastic_shards, reload_enabled=reload_enabled,
            kafka_port=kafka_port, kafka_ip=kafka_ip, metricbeat_modules=metricbeat_modules)
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.configMetricbeat(config_metricbeat_msg, timeout=timeout)
    return host_dto


def start_heartbeat(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to start heartbeat

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    start_heartbeat_msg = csle_collector.host_manager.host_manager_pb2.StartHeartbeatMsg()
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.startHeartbeat(start_heartbeat_msg, timeout=timeout)
    return host_dto


def stop_heartbeat(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to stop heartbeat

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    stop_heartbeat_msg = \
        csle_collector.host_manager.host_manager_pb2.StopHeartbeatMsg()
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.stopHeartbeat(stop_heartbeat_msg, timeout=timeout)
    return host_dto


def config_heartbeat(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        kibana_ip: str, kibana_port: int, elastic_ip: str,
        elastic_port: int, num_elastic_shards: int,
        hosts_to_monitor: List[str], timeout=constants.GRPC.CONFIG_TIMEOUT_SECONDS) \
        -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to update the heartbeat configuration

    :param kibana_ip: the IP of Kibana where the data should be visualized
    :param kibana_port: the port of Kibana where the data should be visualized
    :param elastic_ip: the IP of elastic where the data should be shipped
    :param elastic_port: the port of elastic where the data should be shipped
    :param num_elastic_shards: the number of elastic shards
    :param hosts_to_monitor: list of hosts to monitor with heartbeats
    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    config_heartbeat_msg = \
        csle_collector.host_manager.host_manager_pb2.ConfigHeartbeatMsg(
            kibana_ip=kibana_ip, kibana_port=kibana_port, elastic_ip=elastic_ip,
            elastic_port=elastic_port, num_elastic_shards=num_elastic_shards, hosts_to_monitor=hosts_to_monitor)
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.configHeartbeat(config_heartbeat_msg, timeout=timeout)
    return host_dto


def start_spark(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to start spark

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    start_spark_msg = csle_collector.host_manager.host_manager_pb2.StartSparkMsg()
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.startSpark(start_spark_msg, timeout=timeout)
    return host_dto


def stop_spark(
        stub: csle_collector.host_manager.host_manager_pb2_grpc.HostManagerStub,
        timeout=constants.GRPC.TIMEOUT_SECONDS) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
    """
    Sends a request to the Host manager to stop spark

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a HostDTO describing the host status
    """
    stop_spark_msg = \
        csle_collector.host_manager.host_manager_pb2.StopSparkMsg()
    host_dto: csle_collector.host_manager.host_manager_pb2.HostStatusDTO = \
        stub.stopSpark(stop_spark_msg, timeout=timeout)
    return host_dto
